/*
Title: i3Geo

Arquivo:

i3geo/classesjs/classe_i3geo.js

Licenca:

GPL2

I3Geo Interface Integrada de Ferramentas de Geoprocessamento para Internet

Direitos Autorais Reservados (c) 2006 Ministrio do Meio Ambiente Brasil
Desenvolvedor: Edmar Moretti edmar.moretti@mma.gov.br

Este programa  software livre; voc pode redistribu-lo
e/ou modific-lo sob os termos da Licena Pblica Geral
GNU conforme publicada pela Free Software Foundation;
tanto a verso 2 da Licena.
Este programa  distribudo na expectativa de que seja til,
porm, SEM NENHUMA GARANTIA; nem mesmo a garantia implcita
de COMERCIABILIDADE OU ADEQUAO A UMA FINALIDADE ESPECFICA.
Consulte a Licena Pblica Geral do GNU para mais detalhes.
Voc deve ter recebido uma cpia da Licena Pblica Geral do
GNU junto com este programa; se no, escreva para a
Free Software Foundation, Inc., no endereo
59 Temple Street, Suite 330, Boston, MA 02111-1307 USA.
*/
/*
Classe: i3GEO

A classe i3GEO possu os mtodos de criao e atualizao do mapa. Todas as subclasses
so baseadas em i3GEO, por exemplo, para criar uma janela flutuante sobre o mapa,
utilize i3GEO.janela.cria()

Para inicializar o mapa, utilize i3GEO.inicia() e para atualizar o mapa, utilize i3GEO.atualiza().
Aps terminado o processo de inicializao, pode-se executar uma funo de ajuste. Essa funo
deve ser definida em i3GEO.finaliza, por exemplo i3GEO.finaliza("funcaoDeAjuste()")

Ao inicializar ou atualizar o i3Geo,  feita uma chamada em AJAX 
para a obteno dos parmetros necessrios ao funcionamento do mapa. Esses parmetros
so armazenados na varivel i3GEO.parametros

Nessa classe esto disponveis variveis internas utilizadas em vrias funes, como i3GEO.temaAtivo
*/
i3GEO = {
	
	/*
	Propriedade: parametros
	
	Parmetros obtidos do mapa atual. Os parmetros so fornecidos pelos programas
	PHP de redesenho e criao do mapa e atualizados sempre que o mapa  alterado.
	
	Exemplos:
	
	alert(i3GEO.parametros.mapexten);
	i3GEO.mapscale = 25000;
	
	Parametros:
	
	mapexten {String} - extenso geogrfica do mapa no formato xmin ymin xmax ymax
	
	mapscale {Numeric} - denominador da escala do mapa
	
	mapres {Numeric} - resoluo da imagem do mapa em DPI
	
	pixelsize {Numeric} - tamanho em unidades de terreno dos pixels da imagem
	
	mapfile {String} - nome do mapfile atualmente em uso
	
	cgi {String} - endereo do executvel do mapserver no servidor acessvel pela URL
	
	extentTotal {String} - extenso do mapa na inicializao
	
	mapimagem {String} - URL da imagem que compe o mapa
	
	geoip {sim|nao} - indica se o geoip est instalado
	
	listavisual {String} - lista de visuais disponveis
	
	utilizacgi {sim|nao} - indica se o mapa atual est no modo CGI
	
	versaoms {String} - verso do Mapserver instalado no servidor (maior)
	
	versaomscompleta {String} - verso do Mapserver instalado no servidor (completa)
	
	mensagens {String} - mensagens para uso no letreiro
	
	w {Integer} - largura do mapa atual
	
	h {Integer} - altura do mapa atual
	
	locsistemas {String} - endereo do xml com a lista de sistemas adicionais
	
	locidentifica {String} - endereo do xml com a lista de sistemas de identificao
	
	r {sim|nao} - indica se o software R est instalado no servidor
	
	locmapas {String} - endereo do xml com a lista de mapas
	
	extentref {String} - extenso geogrfica do mapa de referncia
	
	celularef {Numeric} - tamanho do pixel do mapa de referncia em unidades do terreno
	
	kmlurl {String} - url de um arquivo kml que ser inserido no mapa. Vlido apenas na interface Google Maps
	*/
	parametros: {
		mapexten: "",
		mapscale: "",
		mapres: "",
		pixelsize: "",
		mapfile: "",
		cgi: "",
		extentTotal: "",
		mapimagem: "",
		geoip: "",
		listavisual: "",
		utilizacgi:"",
		versaoms:"",
		versaomscompleta: "",
		mensagens:"",
		w: "",
		h: "",
		locsistemas:"",
		locidentifica:"",
		r:"",
		locmapas:"",
		celularef:"",
		kmlurl:""
	},
	/*
	Propriedade: finaliza
	
	Armazena o nome de uma funo que ser executada aps a inicializao do mapa
	
	Tipo:
	{string}
	
	Default:
	{""}
	*/
	finaliza:"",
	/*
	Variavel: temaAtivo
	
	Indica o ltimo tema que foi ativado no mapa
	
	Permite que ao se passar de uma ferramenta
	para outra, os menus reflitam a ltima escolha.
	
	Tipo:
	{string}
	
	Default:
	{""}
	*/
	temaAtivo: "",
	/*
	Function: cria
	
	Cria e configura a visualizao do mapa, definindo posicionamentos, tamanho etc
	
	Aps as definies bsicas,  executado o programa <i3GEO.interface.cria> que ir
	realizar as operaes conforme a interface atualmente em uso. A interface  definida
	em <i3GEO.interface.ATUAL>
	
	Veja <i3GEO.interface>
	*/
	cria:function(){
		if (window.location.href.split("?")[1]){
			i3GEO.configura.sid = window.location.href.split("?")[1];
			g_sid = i3GEO.configura.sid;
			//
			//a biblioteca YUI, por algum motivo, acrescenta # na URL. O # precisa ser removido, caso contrrio, a opo de reload da pgina pelo browser as vezes no funciona
			//
			if (i3GEO.configura.sid.split("#")[0])
			{i3GEO.configura.sid = i3GEO.configura.sid.split("#")[0];}
		}
		else
		{i3GEO.configura.sid = "";}
		//para efeitos de compatibilidade
		g_panM = "nao";
		try {i3GEO.configura.locaplic = g_locaplic;}
		catch(e){g_locaplic = i3GEO.configura.locaplic;};	
		try{i3GEO.configura.diminuixM = g_diminuixM;}catch(e){}
		try{i3GEO.configura.diminuixN = g_diminuixN;}catch(e){}
		try{i3GEO.configura.diminuiyM = g_diminuiyM;}catch(e){}
		try{i3GEO.configura.diminuiyN = g_diminuiyN;}catch(e){}			
		//
		//calcula o tamanho do mapa
		//
		var diminuix = (navm) ? i3GEO.configura.diminuixM : i3GEO.configura.diminuixN;
		var diminuiy = (navm) ? i3GEO.configura.diminuiyM : i3GEO.configura.diminuiyN;
		//
		//subtrai barra de rolagem
		//
		try{
			var a = i3GEO.util.getScrollerWidth();
			var diminuiy = diminuiy + a;
		}
		catch(r){}
		var menos = 0;
		if ($i("contemFerramentas"))
		{var menos = menos + parseInt($i("contemFerramentas").style.width);}
		if ($i("ferramentas"))
		{var menos = menos + parseInt($i("ferramentas").style.width);}
		var novow = parseInt(screen.availWidth) - diminuix;
		var novoh = parseInt(screen.availHeight) - diminuiy;
		if (window.top==window.self){//nao se aplica em iframe		
			window.resizeTo(screen.availWidth,screen.availHeight);
			window.moveTo(0,0);
		}
		//o try aqui  necessrio por conta do uso possvel do i3geo em um iframe
		try{
			if (novow < 800){
				var novow = 800;
				var novoh = 600;
			}
		}
		catch(e){var e = "";}
		document.body.style.width = novow - diminuix;
		document.body.style.height = novoh;
		var w = novow - menos - diminuix;
		var h = novoh - diminuiy;
		var temp = $i("corpoMapa");
		if (temp){
			if(temp.style){
				if (temp.style.width){
					var w = parseInt(temp.style.width);
					var h = parseInt(temp.style.width);
				}
				if (temp.style.height)
				{var h = parseInt(temp.style.height);}
			}
		}
		if($i("contemImg")){
			$i("contemImg").style.height=h + "px";
			$i("contemImg").style.width=w + "px";
		}
		i3GEO.interface.cria(w,h);
		i3GEO.parametros = {
			mapexten: "",
			mapscale: "",
			mapres: "",
			pixelsize: "",
			mapfile: "",
			cgi: "",
			extentTotal: "",
			mapimagem: "",
			geoip: "",
			listavisual: "",
			utilizacgi:"",
			versaoms:"",
			versaomscompleta:"",
			mensagens:"",
			w: w,
			h: h,
			locsistemas:"",
			locidentifica:"",
			r:"",
			locmapas:"",
			extentref:"",
			kmlurl:""
		};
		if(w < 550){
			var i = $i(i3GEO.gadgets.PARAMETROS.mostraQuadros.idhtml);
			if(i){i.style.display = "none"};
		}
	},
	/*
	Function: inicia
	
	Inicializa o mapa aps ter sido criado com i3GEO.cria()
	
	Verifica se o mapa apresenta algum problema no processamento no lado do servidor e
	realiza as operaes de tentativa de recuperao, se for o caso
	
	No incio do processo  executada a funo <i3GEOmantemCompatibilidade>
	para realizar as operaes necessrias de manuteno de compatibilidade da verso atual para as anteriores
	
	A inicializao  baseada em <i3GEO.php.inicia> cujo retorno  utilizado para definir a
	varivel <i3GEO.parametros>
	
	Aps a inicializao  executado <i3GEO.interface.inicia>
	*/
	inicia:function(){
		if(typeof("i3GEOmantemCompatibilidade") == 'function')
		i3GEOmantemCompatibilidade();
		var montaMapa = function(retorno){
			if(retorno == ""){alert("Ocorreu um erro no mapa - montaMapa");retorno = {data:{erro: "erro"}};}
			if(retorno.data.erro){
				i3GEO.janela.fechaAguarde("montaMapa");
				document.body.style.backgroundColor="white";
				document.body.innerHTML = "<br>Para abrir o i3Geo utilize o link:<br><a href="+i3GEO.configura.locaplic+"/ms_criamapa.php >"+i3GEO.configura.locaplic+"/ms_criamapa.php</a>";
				return("linkquebrado");
			}
			else{
				if(retorno.data.variaveis){
					//
					//executa com eval a string que  retornada pelo servidor (funo inicia do mapa_controle.php
					//
					var tempo = "";
					var titulo = "";
					eval(retorno.data.variaveis);
					try{
						if (titulo != "")
						{top.document.title = titulo;}
					}
					catch(e){var e = "";}
					i3GEO.ajuda.mostraJanela("Tempo de desenho em segundos: "+tempo,"");
					i3GEO.parametros.mapexten= mapexten;
					i3GEO.parametros.mapscale= parseInt(mapscale);
					i3GEO.parametros.mapres= mapres;
					i3GEO.parametros.pixelsize= g_celula;
					i3GEO.parametros.mapfile= mapfile;
					i3GEO.parametros.cgi= cgi;
					i3GEO.parametros.extentTotal=mapexten;
					i3GEO.parametros.mapimagem= mapimagem;
					i3GEO.parametros.geoip= geoip;
					i3GEO.parametros.listavisual= listavisual;
					i3GEO.parametros.utilizacgi= utilizacgi;
					i3GEO.parametros.versaoms= versaoms;
					i3GEO.parametros.mensagens= mensagens;
					i3GEO.parametros.locsistemas = locsistemas;
					i3GEO.parametros.locidentifica = locidentifica;
					i3GEO.parametros.r = r;
					i3GEO.parametros.locmapas = locmapas;
					i3GEO.parametros.extentref = extentref;
					i3GEO.parametros.versaoms = versaoms;
					i3GEO.parametros.versaomscompleta = versaomscompleta;
					i3GEO.parametros.kmlurl = kmlurl;
					
					i3GEO.gadgets.quadros.inicia(10);
					i3GEO.gadgets.quadros.grava("extensao",mapexten);
					
					i3GEO.arvoreDeCamadas.cria("",retorno.data.temas,i3GEO.configura.sid,i3GEO.configura.locaplic);
					i3GEO.util.arvore("<b>"+$trad("p13")+"</b>","listaPropriedades",i3GEO.configura.listaDePropriedadesDoMapa);

					i3GEO.gadgets.mostraBuscaRapida();
					i3GEO.guias.cria();
					if($i("arvoreAdicionaTema"))
					i3GEO.arvoreDeTemas.cria(i3GEO.configura.sid,i3GEO.configura.locaplic,"arvoreAdicionaTema");
					
					if($i("mst")){$i("mst").style.display="block";}
					i3GEO.atualiza(retorno);
					//
					//calcula (opcional) o tamanho correto da tabela onde fica o mapa
					//se no for feito esse clculo, o mapa fica ajustado  esquerda
					//			
					var temp = 0;
					if ($i("contemFerramentas")){temp = temp + parseInt($i("contemFerramentas").style.width);}
					if ($i("ferramentas")){temp = temp + parseInt($i("ferramentas").style.width);}
					if($i("mst"))
					{$i("mst").style.width=i3GEO.parametros.w + temp + "px";}
					if (i3GEO.configura.entorno == "sim"){
						i3GEO.configura.entorno == "nao";
						i3GEO.navega.entorno.ativaDesativa();
					}
					i3GEO.navega.autoRedesenho.ativa();
					if ($i("i3geo_escalanum")){$i("i3geo_escalanum").value = i3GEO.parametros.mapscale;}
					if ((i3GEO.parametros.geoip == "nao") && ($i("ondeestou")))
					{$i("ondeestou").style.display="none";}
					i3GEO.interface.inicia();
				}
				else
				{alert("Erro. Impossivel criar o mapa "+retorno.data);return;}
				//
				//ativa a janela de mensagens se for o caso
				//
				if(document.getElementById("ajuda")) //para efeitos de compatibilidade com as verses anteriores a 4.1
				{i3GEO.ajuda.DIVAJUDA = "ajuda";}
				var abreJM = "sim";
				if (i3GEO.util.pegaCookie("g_janelaMen")){
					var abreJM = i3GEO.util.pegaCookie("g_janelaMen");
					if(abreJM == "sim")
					i3GEO.configura.iniciaJanelaMensagens = true;
					else
					i3GEO.configura.iniciaJanelaMensagens = false;
				}
				if(i3GEO.configura.iniciaJanelaMensagens == true)
				{i3GEO.ajuda.abreJanela();}		
				i3GEO.janela.fechaAguarde("montaMapa");
				if (i3GEO.configura.liberaGuias == "sim"){i3GEO.guias.libera();}
			}
			if($i("mst")){$i("mst").style.visibility ="visible";}	
		};
		if (!$i("i3geo"))
		{document.body.id = "i3geo";}
		$i("i3geo").className = "yui-skin-sam";
		if($i("mst"))
		$i("mst").style.visibility ="hidden";
		//
		//se g_sid="", o html foi aberto diretamente
		//ento,  necessrio criar os arquivos temporrios do mapa
		//essa operao deve ser assncrona
		//
		if(i3GEO.configura.sid==""){
			var mashup = function (retorno){
				i3GEO.configura.sid = retorno.data;
				i3GEO.inicia();
			};
			i3GEO.php.criamapa(mashup,i3GEO.configura.mashuppar);
		}
		else{
			//YAHOO.log("Chamada AJAX para obter o mapa inicial", "i3geo");
			i3GEO.janela.abreAguarde("montaMapa",$trad("o5"));
			i3GEO.php.inicia(montaMapa,i3GEO.configura.embedLegenda,i3GEO.parametros.w,i3GEO.parametros.h);
		}
		if(i3GEO.eventos.NAVEGAMAPA.toString().search("i3GEO.janela.fechaAguarde()") < 0)
		{i3GEO.eventos.NAVEGAMAPA.push("i3GEO.janela.fechaAguarde()");}
		eval(i3GEO.finaliza);
	},
	/*
	Function: atualiza
	
	Atualiza o mapa atual, altera a imagem do mapa os gadgets ativos e os parmetros e
	verifica a integridade do mapa em uso (arquivo mapfile)
	
	O processo executa tambm a funo de atualizao especfica da interface atual em uso, veja
	<i3GEO.interface.redesenha>
	
	Os seguintes gadgets so processados
	
	<i3GEO.arvoreDeCamadas.atualiza>
	
	<i3GEO.arvoreDeCamadas.atualizaFarol>
	
	Os eventos definidos em <i3GEO.eventos.navegaMapa> so executados
	
	Parametro:
	
	retorno {String} - string com os parmetros do novo mapa. Se retorno no
	for especificado ou se for vazio, ser feita uma chamada em ajax para sua obteno. O resultado
	dessa chamada  armazenada em i3GEO.parametros
	*/
	atualiza: function(retorno){
		var corpoMapa = function(){
			i3GEO.janela.abreAguarde("ajaxiniciaParametros",$trad("o1")+" atualizando");
			i3GEO.php.corpo(i3GEO.atualiza,i3GEO.configura.tipoimagem);		
		};
		if(arguments.length == 0){
			corpoMapa.call();
			return;
		}
		if(retorno == ""){
			corpoMapa.call();
			return;
		}
		if(!retorno.data){
			corpoMapa.call();
			return;
		}
		//verifica se o parmetro retorno existe, caso contrrio,
		//faz a chamada ao programa PHP para obter os parmetros
		try{
			if (retorno.data == "erro"){
				alert("Erro no mapa. Sera feita uma tentativa de recuperacao.");
				i3GEO.mapa.recupera.inicia();return;
			}
			else
			if(retorno.data == "ok" || retorno.data == ""){corpoMapa.call();return;}
		}
		catch(e){}
		var erro = function(){
			var legimagem = "";
			var c = confirm("Ocorreu um erro, quer tentar novamente?");
			if(c){
				corpoMapa.call();
			}
			else{
				i3GEO.janela.fechaAguarde();
			}
			return;
		};
		try{eval(retorno.data.variaveis);}
		catch(e){erro.call();return;}
		if(arguments.length == 0 || retorno == "" || retorno.data.variaveis == undefined){erro.call();return;}
		else{	
			if(arguments.length == 0){return;}
			i3GEO.mapa.verifica(retorno);
			var tempo = "";
			if(i3GEO.desenho.richdraw)
			{i3GEO.desenho.richdraw.clearWorkspace();}
			mapscale = "";
			mapexten = "";
			//transforma o retorno em variveis
			eval(retorno.data.variaveis);
			//
			//o try aqui  necessrio pois na interface googlemaps os parmetros retorno.data.variaveis no so gerados completamente
			//
			try{
				i3GEO.arvoreDeCamadas.atualiza(retorno.data.temas);
				if (i3GEO.parametros.mapscale != mapscale)
				i3GEO.arvoreDeCamadas.atualizaFarol(mapscale);
				i3GEO.parametros.mapexten = mapexten;
				i3GEO.parametros.mapscale = mapscale;
				i3GEO.parametros.mapres = mapres;
				i3GEO.parametros.pixelsize = g_celula;
				i3GEO.parametros.mapimagem = mapimagem;
			}
			catch(e){}
			i3GEO.interface.redesenha();
			
			g_operacao = "";
			i3GEO.parametros.mapexten = mapexten;
			if ($i("mensagemt"))
			{$i("mensagemt").value = i3GEO.parametros.mapexten;}
			
			i3GEO.arvoreDeCamadas.CAMADAS = retorno.data.temas;
			i3GEO.eventos.navegaMapa();
			if (i3GEO.configura.entorno == "sim"){
				i3GEO.navega.entorno.geraURL();
				i3GEO.navega.entorno.ajustaPosicao();
			}
			i3GEO.ajuda.mostraJanela("Tempo de redesenho em segundos: "+tempo,"");
		}	
	}
};
//YAHOO.log("carregou classe i3geo", "Classes i3geo");